@@ -29,7 +29,32 @@ pub fn gen(def: &TypeDef, gen: &Gen) -> TokenStream {
         fields.clear();
     }
 
-    let mut tokens = if is_scoped {
+    let eq = if gen.sys {
+        quote! {}
+    } else {
+        quote! {
+            // Unfortunately, Rust requires these to be derived to allow constant patterns.
+            #[derive(::core::cmp::PartialEq, ::core::cmp::Eq)]
+        }
+    };
+
+    let mut tokens = if is_scoped || !gen.sys {
+        quote! {
+            #doc
+            #features
+            #[repr(transparent)]
+            #eq
+            pub struct #ident(pub #underlying_type);
+        }
+    } else {
+        quote! {
+            #doc
+            #features
+            pub type #ident = #underlying_type;
+        }
+    };
+
+    tokens.combine(&if is_scoped {
         let fields = fields.iter().map(|(field_name, value)| {
             quote! {
                 pub const #field_name: Self = Self(#value);
@@ -37,22 +62,22 @@ pub fn gen(def: &TypeDef, gen: &Gen) -> TokenStream {
         });
 
         quote! {
-            #doc
-            #features
-            #[repr(transparent)]
-            pub struct #ident(pub #underlying_type);
             #features
             impl #ident {
                 #(#fields)*
             }
-            #features
-            impl ::core::marker::Copy for #ident {}
-            #features
-            impl ::core::clone::Clone for #ident {
-                fn clone(&self) -> Self {
-                    *self
-                }
+        }
+    } else if !gen.sys {
+        let fields = fields.iter().map(|(field_name, value)| {
+            quote! {
+                #doc
+                #features
+                pub const #field_name: #ident = #ident(#value);
             }
+        });
+
+        quote! {
+            #(#fields)*
         }
     } else {
         let fields = fields.iter().map(|(field_name, value)| {
@@ -64,71 +89,82 @@ pub fn gen(def: &TypeDef, gen: &Gen) -> TokenStream {
         });
 
         quote! {
-            #doc
-            #features
-            pub type #ident = #underlying_type;
             #(#fields)*
         }
-    };
+    });
+
+    if is_scoped || !gen.sys {
+        tokens.combine(&quote! {
+            #features
+            impl ::core::marker::Copy for #ident {}
+            #features
+            impl ::core::clone::Clone for #ident {
+                fn clone(&self) -> Self {
+                    *self
+                }
+            }
+        });
+    }
 
     if !gen.sys {
-        if is_scoped {
-            tokens.combine(&quote! {
-                #features
-                unsafe impl ::windows::core::Abi for #ident {
-                    type Abi = Self;
+        tokens.combine(&quote! {
+            #features
+            impl ::core::default::Default for #ident {
+                fn default() -> Self {
+                    Self(0)
                 }
-                #features
-                impl ::core::cmp::PartialEq for #ident {
-                    fn eq(&self, other: &Self) -> bool {
-                        self.0 == other.0
+            }
+        });
+    }
+
+    if !gen.sys {
+        tokens.combine(&quote! {
+            #features
+            unsafe impl ::windows::core::Abi for #ident {
+                type Abi = Self;
+            }
+            #features
+            impl ::core::fmt::Debug for #ident {
+                fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
+                    f.debug_tuple(#name).field(&self.0).finish()
+                }
+            }
+        });
+
+        if def.has_flags() {
+            tokens.combine(&quote! {
+                impl ::core::ops::BitOr for #ident {
+                    type Output = Self;
+
+                    fn bitor(self, other: Self) -> Self {
+                        Self(self.0 | other.0)
                     }
                 }
-                #features
-                impl ::core::cmp::Eq for #ident {}
-                #features
-                impl ::core::fmt::Debug for #ident {
-                    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-                        f.debug_tuple(#name).field(&self.0).finish()
+                impl ::core::ops::BitAnd for #ident {
+                    type Output = Self;
+
+                    fn bitand(self, other: Self) -> Self {
+                        Self(self.0 & other.0)
+                    }
+                }
+                impl ::core::ops::BitOrAssign for #ident {
+                    fn bitor_assign(&mut self, other: Self) {
+                        self.0.bitor_assign(other.0)
+                    }
+                }
+                impl ::core::ops::BitAndAssign for #ident {
+                    fn bitand_assign(&mut self, other: Self) {
+                        self.0.bitand_assign(other.0)
+                    }
+                }
+                impl ::core::ops::Not for #ident {
+                    type Output = Self;
+
+                    fn not(self) -> Self {
+                        Self(self.0.not())
                     }
                 }
             });
-
-            if def.has_flags() {
-                tokens.combine(&quote! {
-                    impl ::core::ops::BitOr for #ident {
-                        type Output = Self;
-
-                        fn bitor(self, other: Self) -> Self {
-                            Self(self.0 | other.0)
-                        }
-                    }
-                    impl ::core::ops::BitAnd for #ident {
-                        type Output = Self;
-
-                        fn bitand(self, other: Self) -> Self {
-                            Self(self.0 & other.0)
-                        }
-                    }
-                    impl ::core::ops::BitOrAssign for #ident {
-                        fn bitor_assign(&mut self, other: Self) {
-                            self.0.bitor_assign(other.0)
-                        }
-                    }
-                    impl ::core::ops::BitAndAssign for #ident {
-                        fn bitand_assign(&mut self, other: Self) {
-                            self.0.bitand_assign(other.0)
-                        }
-                    }
-                    impl ::core::ops::Not for #ident {
-                        type Output = Self;
-
-                        fn not(self) -> Self {
-                            Self(self.0.not())
-                        }
-                    }
-                });
-            }
         }
 
         if def.is_winrt() {
@@ -145,6 +181,8 @@ pub fn gen(def: &TypeDef, gen: &Gen) -> TokenStream {
                 }
             });
         }
+
+        tokens.combine(&extensions::gen(def));
     }
 
     tokens
