@@ -10,8 +10,10 @@ mod functions;
 mod gen;
 mod handles;
 mod helpers;
+mod implements;
 mod interfaces;
 mod iterator;
+mod method_names;
 mod methods;
 mod names;
 mod replacements;
@@ -23,18 +25,19 @@ use functions::*;
 pub use gen::*;
 use helpers::*;
 use iterator::*;
+use metadata::*;
+use method_names::*;
 use methods::*;
 use names::*;
-use quote::*;
 use r#async::*;
-use reader::*;
 use signatures::*;
+use tokens::*;
 
 pub fn gen_type(name: &str, gen: &Gen) -> String {
     let reader = TypeReader::get();
     let mut tokens = String::new();
 
-    for def in reader.get_type_entry(TypeName::parse(name)).iter().flat_map(|entry| entry.def.iter()) {
+    for def in reader.get_type_entry(TypeName::parse(name)).iter().flat_map(|entry| entry.iter()) {
         tokens.push_str(gen_element_type(def, gen).as_str());
     }
 
@@ -70,11 +73,27 @@ pub fn gen_namespace(gen: &Gen) -> String {
     tokens.into_string()
 }
 
+pub fn gen_namespace_impl(gen: &Gen) -> String {
+    let tree = TypeReader::get().get_namespace(gen.namespace).expect("Namespace not found");
+    let mut tokens = TokenStream::new();
+
+    for entry in tree.types.values() {
+        for def in entry {
+            if let ElementType::TypeDef(def) = def {
+                let def = &def.clone().with_generics();
+                tokens.combine(&implements::gen(def, gen));
+            }
+        }
+    }
+
+    tokens.into_string()
+}
+
 fn gen_non_sys_function_types(tree: &TypeTree, gen: &Gen) -> TokenStream {
     let mut tokens = TokenStream::new();
 
     for entry in tree.types.values() {
-        for def in &entry.def {
+        for def in entry {
             tokens.combine(&gen_element_type(def, gen));
         }
     }
