@@ -27,6 +27,7 @@ fn gen_struct_with_name(def: &TypeDef, struct_name: &str, cfg: &Cfg, gen: &Gen)
             let guid = gen_element_name(&ElementType::GUID, gen);
             return quote! { pub const #name: #guid = #value; };
         } else if name.as_str().ends_with("Vtbl") {
+            // This just omits some useless struct declarations like `IDDVideoPortContainerVtbl`
             return quote! {};
         } else {
             return quote! {
@@ -48,7 +49,10 @@ fn gen_struct_with_name(def: &TypeDef, struct_name: &str, cfg: &Cfg, gen: &Gen)
 
     let fields = def.fields().map(|f| {
         let name = gen_ident(f.name());
-        let sig = gen_sig(&f.signature(Some(def)), gen);
+        let mut sig = f.signature(Some(def));
+        sig.is_const = sig.is_const || f.is_const();
+        let sig = gen_sig(&sig, gen);
+
         if f.is_literal() {
             quote! {}
         } else if !gen.sys && is_union && !f.is_blittable(Some(def)) {
@@ -190,7 +194,7 @@ fn gen_compare_traits(def: &TypeDef, name: &TokenStream, cfg: &Cfg, gen: &Gen) -
 }
 
 fn gen_debug(def: &TypeDef, ident: &TokenStream, cfg: &Cfg, gen: &Gen) -> TokenStream {
-    if gen.sys || def.is_union() || def.has_explicit() || def.is_packed() {
+    if gen.sys || def.has_union() || def.has_pack() {
         quote! {}
     } else {
         let name = ident.as_str();
